/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace RhodesProcessorLib
{
    public class CollisionInfo
    {
        public enum CollisionType
        {
            Rectangle = 0,
            Circle = 1,
            Polygon = 2
        }

        private CollisionType _type;
        private Vector2 _offset;
        private float _rotationOffset;
        
        public CollisionInfo(CollisionType type) 
        {
            _type = type;
            _offset = new Vector2();
            _rotationOffset = 0.0f;
        }

        public Vector2 Offset { get { return _offset; } set { _offset = value; } }
        public float RotationOffset { get { return _rotationOffset; } set { _rotationOffset = value; } }
        public CollisionType Type { get { return _type; } set { _type = value; } }
    }

    public class RectangleCollisionInfo : CollisionInfo
    {
        private float _width;
        private float _height;

        public RectangleCollisionInfo() : base(CollisionInfo.CollisionType.Rectangle) 
        {
            _width = 25;
            _height = 25;
        }

        public float Width { get { return _width; } set { _width = value; } }
        public float Height { get { return _height; } set { _height = value; } }
    }

    public class CircleCollisionInfo : CollisionInfo
    {
        private float _radius;
        private int _edges;

        public CircleCollisionInfo() : base(CollisionInfo.CollisionType.Circle) { }

        public float Radius { get { return _radius; } set { _radius = value; } }
        public int Edges { get { return _edges; } set { _edges = value; } }
    }

    public class PolygonCollisionInfo : CollisionInfo
    {
        private List<Vector2> _vertices;
        private float _collisionGridCellSize;

        public PolygonCollisionInfo() : base(CollisionInfo.CollisionType.Polygon) { _vertices = new List<Vector2>(); }
        public PolygonCollisionInfo(List<Vector2> vertices) : base(CollisionInfo.CollisionType.Polygon) { _vertices = vertices; }

        public float CollisionGridCellSize { get { return _collisionGridCellSize; } set { _collisionGridCellSize = value; } }
        public List<Vector2> Vertices { get { return _vertices; } set { _vertices = value; } }        
    }
}
