/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content.Pipeline;
using System.Xml;

namespace RhodesProcessorLib
{
    /*! Custom content processor for CompoundEntity */
    [ContentProcessor]
    class CompoundEntityProcessor : ContentProcessor<CompoundEntitySource, CompoundEntityCompiled>
    {
        public override CompoundEntityCompiled Process(CompoundEntitySource input, ContentProcessorContext context)
        {
            CompoundEntityCompiled retVal = new CompoundEntityCompiled();
            XmlDocument xml = input.Xml;

            foreach (XmlNode node in xml.GetElementsByTagName("general")[0].ChildNodes)
            {
                if (node.Name.Equals("name"))
                    retVal.Name = node.InnerText;
                if (node.Name.Equals("simple"))
                    retVal.Entities.Add(node.InnerText);
                if (node.Name.Equals("active"))
                    retVal.Active = Boolean.Parse(node.InnerText);
            }

            foreach (XmlNode node in xml.GetElementsByTagName("joint")[0].ChildNodes)
            {
                if (node.Name.Equals("revoluteJoint"))
                {
                    JointInfo jointInfo = new RevoluteJointInfo();

                    foreach (XmlNode info in node.ChildNodes)
                    {
                        if (info.Name.Equals("entity1"))
                            jointInfo.Entity1 = info.InnerText;
                        if (info.Name.Equals("entity2"))
                            jointInfo.Entity2 = info.InnerText;
                        if (info.Name.Equals("offset"))
                        {
                            String[] point = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                            ((RevoluteJointInfo)jointInfo).Offset = new Vector2(float.Parse(point[0]), float.Parse(point[1]));
                        }
                        if (info.Name.Equals("biasFactor"))
                            jointInfo.BiasFactor = float.Parse(info.InnerText);
                        if (info.Name.Equals("softness"))
                            jointInfo.Softness = float.Parse(info.InnerText);
                    }

                    retVal.RevoluteJoints.Add(jointInfo);
                }

                if (node.Name.Equals("angleJoint"))
                {
                    JointInfo jointInfo = new AngleJointInfo();

                    foreach (XmlNode info in node.ChildNodes)
                    {
                        if (info.Name.Equals("entity1"))
                            jointInfo.Entity1 = info.InnerText;
                        if (info.Name.Equals("entity2"))
                            jointInfo.Entity2 = info.InnerText;
                        if (info.Name.Equals("targetAngle"))
                            ((AngleJointInfo)jointInfo).TargetAngle = float.Parse(info.InnerText);
                        if (info.Name.Equals("maxImpulse"))
                            ((AngleJointInfo)jointInfo).MaxImpulse = float.Parse(info.InnerText);
                        if (info.Name.Equals("biasFactor"))
                            jointInfo.BiasFactor = float.Parse(info.InnerText);
                        if (info.Name.Equals("softness"))
                            jointInfo.Softness = float.Parse(info.InnerText);
                    }

                    retVal.AngleJoints.Add(jointInfo);
                }
            }

            return retVal;
        }
    }
}
