using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content.Pipeline;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;
using System.Xml;
using rhodes.graphics;

namespace SedohrProcessorLib
{
    [ContentProcessor]
    class LevelProcessor : ContentProcessor<LevelSource, LevelCompiled>    
    {
        public override LevelCompiled Process(LevelSource input, ContentProcessorContext context)
        {
            LevelCompiled retVal = new LevelCompiled();
            XmlDocument xml = input.Xml;

            foreach (XmlNode node in xml.GetElementsByTagName("level")[0].ChildNodes)
            {
                LayerInfo layerInfo = new LayerInfo();
                layerInfo.Depth = int.Parse(node.Attributes["depth"].InnerText);
                layerInfo.Proportion = float.Parse(node.Attributes["proportion"].InnerText);

                foreach (XmlNode layer in node.ChildNodes)
                {
                    if (layer.Name.Equals("static"))
                    {
                        layerInfo.Statics.Add(layer.InnerText);
                    }
                    if (layer.Name.Equals("simple-object"))
                    {
                        layerInfo.SimpleObjects.Add(layer.InnerText);
                    }
                    if (layer.Name.Equals("compound-object"))
                    {
                        layerInfo.CompoundObjects.Add(layer.InnerText);
                    }
                    if (layer.Name.Equals("light"))
                    {
                        LightInfo light = new LightInfo();
                        foreach (XmlNode lightInfo in layer.ChildNodes)
                        {
                            if (lightInfo.Name.Equals("position"))
                            {
                                String[] point = lightInfo.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                                light.Position = new Vector3(float.Parse(point[0]), float.Parse(point[1]), float.Parse(point[2]));
                            }
                            if (lightInfo.Name.Equals("attenuation"))
                            {
                                String[] point = lightInfo.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                                light.Attenuation = new Vector3(float.Parse(point[0]), float.Parse(point[1]), float.Parse(point[2]));
                            }
                            if (lightInfo.Name.Equals("color"))
                            {
                                String[] point = lightInfo.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                                light.Color = new Vector4(
                                    float.Parse(point[0]), 
                                    float.Parse(point[1]), 
                                    float.Parse(point[2]),
                                    float.Parse(point[3]));
                            }
                            if (lightInfo.Name.Equals("isAttenuated"))
                            {
                                light.IsAttenuated = bool.Parse(lightInfo.InnerText);
                            }
                        }
                        layerInfo.LightInfo.Add(light);
                    }
                    if (layer.Name.Equals("graphics"))
                    {
                        GraphicsInfo graphicsInfo = new GraphicsInfo();
                        graphicsInfo.Filename = layer.InnerText;
                        graphicsInfo.Position = new Vector2(
                            float.Parse(layer.Attributes["x"].InnerText),
                            float.Parse(layer.Attributes["y"].InnerText));
                        layerInfo.Graphics.Add(graphicsInfo);
                    }
                }

                retVal.Layers.Add(layerInfo);
            }

            return retVal;
        }
    }
}
