uniform extern texture diffuseTexture;
sampler diffuseSampler = sampler_state
{
    Texture = <diffuseTexture>;
    mipfilter = LINEAR; 
};

uniform extern texture lightTexture;
sampler lightSampler = sampler_state
{
    Texture = <lightTexture>;
    mipfilter = LINEAR; 
};

uniform extern texture normalTexture;
sampler normalSampler = sampler_state
{
    Texture = <normalTexture>;
    mipfilter = LINEAR; 
};

struct VS_OUTPUT
{
    float4 position	: POSITION;
    float2 uv0		: TEXCOORD0;
};

uniform float2 dimension;
uniform float3 lightPos = float3(400, 500, 10);
uniform float3 lightAtten = (1.0, 0.001, 0.0001);
uniform float4 lightColor = (1,1,1,1);
uniform bool isAtten = false;

uniform float3 eye = float3(0.5f, 0.5f, 5.0f);

VS_OUTPUT deferredShading_vs(
    float3 position : POSITION, 
    float2 uv0 : TEXCOORD0 )
{
    VS_OUTPUT output = (VS_OUTPUT)0;

    output.position = float4(position,1);
    output.uv0 = uv0;

    return output;
}

float4 light_ps(VS_OUTPUT vsout) : COLOR
{
	float2 uv0 = vsout.uv0;
	float4 diffuseTex = tex2D(diffuseSampler, uv0);
	float3 n = normalize((tex2D(normalSampler, uv0).xyz - 0.5)*2.0);
	float3 E = normalize(eye);
	float3 position = float3(uv0 * dimension, 0.0f); 
	float4 color = tex2D(lightSampler, uv0);
	
	float3 lightDir = normalize(lightPos - position);	
	float NdotL = max(dot(n, lightDir), 0.0);
	
	if(NdotL > 0.0)
	{
		// Light contribution
		float dist = length(lightPos - position);
		float att = 1.0/(lightAtten.x + lightAtten.y*dist + lightAtten.z*dist*dist);
		
		if( isAtten )
			color += att*lightColor*NdotL;
		else
			color += lightColor*NdotL;
		
		// Specular
		//float3 H = normalize(lightDir + E);
		//float NdotH = max(dot(n, H), 0.0);
		//color += att*lightColor*specularTex*specularTex.a*pow(NdotH, 1.0/power);
	}
	
	return float4(color);	
}

float4 light2_ps(VS_OUTPUT vsout) : COLOR
{
	float3 pos = float3( vsout.uv0 * dimension, 0.0f);
	float3 normal = normalize((tex2D(normalSampler, vsout.uv0).xyz - 0.5)*2.0);
	float4 color = tex2D(lightSampler, vsout.uv0);
	
	float3 lightDir = normalize(lightPos - pos);
	float3 lightCol = lightColor / (0.1f * distance(pos, lightPos));	
	float3 specular = 0;//pow(max(0.0f, dot(normal, halfVec)), 2);

	return float4(color.rgb + (lightCol * max(0, dot(normal, lightDir)) + specular), 1);
}

float4 final_ps(VS_OUTPUT vsout) : COLOR
{
	float2 uv0 = vsout.uv0;
	float4 diffuse = tex2D(diffuseSampler, uv0);
	float4 contrib = tex2D(lightSampler, uv0);
	
	//return contrib;	
	//return ( diffuse * ( contrib + ambient ) );
	return ( diffuse * contrib );
	//return contrib;
}

technique light
{
    pass P0
    {
        vertexShader = compile vs_2_0 deferredShading_vs();
        pixelShader  = compile ps_2_0 light_ps();
    }
}

technique final
{
    pass P0
    {
        vertexShader = compile vs_2_0 deferredShading_vs();
        pixelShader  = compile ps_2_0 final_ps();
    }
}