uniform extern texture colorTexture;
sampler colorSampler = sampler_state
{
    Texture = <colorTexture>;
    mipfilter = LINEAR; 
};

uniform extern texture displaceTexture;
sampler displaceSampler = sampler_state
{
    Texture = <displaceTexture>;
    mipfilter = LINEAR; 
};

uniform float mag = 0.1f;
uniform float2 center = float2( 0.5f, 0.5f );
uniform float growth = 0.25f;

struct VS_OUTPUT
{
    float4 position	: POSITION;
    float2 uv0		: TEXCOORD0;
};
 
VS_OUTPUT genericPost_vs(
    float3 position : POSITION, 
    float2 uv0 : TEXCOORD0 )
{
    VS_OUTPUT output = (VS_OUTPUT)0;

    output.position = float4(position,1);
    output.uv0 = uv0;

    return output;
}

float4 displacement_ps(VS_OUTPUT vsout) : COLOR
{
	float4 output;
	float2 displace = center - vsout.uv0;
	float scale = length(displace);
	float radius = abs(-sin(growth * 8) * growth);
	float range = 0.0f;
	
	if( radius != 0.0f )
		range = clamp(1 - (scale / radius), 0, 1);	
	
	displace *= range * mag;
	output = tex2D(displaceSampler, vsout.uv0);
	
	output.r += displace.x;
	output.g += displace.y;
	
	return output;
}

float4 warp_ps(VS_OUTPUT vsout) : COLOR
{	
	float2 uv0 = vsout.uv0;
	float4 displace = tex2D(displaceSampler, uv0);
	float2 displacement = float2(displace.r, displace.g);  
	
	return tex2D(colorSampler, uv0 + displacement);
}

technique displacement
{
    pass P0
    {
        vertexShader = compile vs_2_0 genericPost_vs();
        pixelShader  = compile ps_2_0 displacement_ps();
    }
}

technique warp
{
    pass P0
    {
        vertexShader = compile vs_2_0 genericPost_vs();
        pixelShader  = compile ps_2_0 warp_ps();
    }
}