/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.graphics;
using rhodes.physics;

namespace rhodes.engine
{
    /*! Simple entity in the engine that's responsible for managing both the physics body and the
     * graphical sprite */
    public class SimpleEntity: Entity
    {
		protected PhysicsBody _physicsBody; //!< physics body
        protected Sprite _sprite; //!< sprite
		protected Vector2 _initialPosition; //!< initial position


        public SimpleEntity(String name, PhysicsBody physicsBody, Sprite sprite) : base(name)
        {
            _physicsBody = physicsBody;
            _sprite = sprite;
        }

        /* \copydoc Entity::update */
        public override void update(float elapsedTime)
        {
            if (!_active)
                return;

            _sprite.update(elapsedTime);
            _sprite.Position = _physicsBody.Position;
            _sprite.Rotation = _physicsBody.Rotation;
        }

        /*! Properties */
        public PhysicsBody PhysicsBody { get { return _physicsBody; } }
        public Sprite Sprite { get { return _sprite; } }
        public Boolean Active
        {
            get
            {
                return _active;
            }
            set
            {
                if (_active != value)
                {
                    _active = value;

                    if (_active)
                    {
                        // make entity visible
                        _sprite.Visible = true;

                        // activate the physicsbody
                        if(_physicsBody.Body != null) {
                            _physicsBody.Body.Enabled = true;
                        }
                    }
                    else
                    {
                        // make entity visible
                        _sprite.Visible = false;

                        // activate the physicsbody
                        if(_physicsBody.Body != null) {
                            _physicsBody.Body.Enabled = false;
                        }
                    }
                }
            }
        }
    }
}
