using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;


/* 
 * FileName: Message.cs
 * Author:  Anthony Reese
 * 
 * Represents a message to be printed to the screen
 */
namespace rhodes.graphics
{
	public class Message
	{
		private string _text;
		private Color _color;
		private Vector2 _position;
		private string _font;
		private float _scale;
		private bool _visible;
		private string _name;

		/// <summary>
		/// Creates a message with a string of text
		/// </summary>
		/// <param name="message"></param>
		public Message(String text, Vector2 position, Color color, string font, string name)
		{
			_text = text;
			_position = position;
			_color = color;
			_font = font;
			_scale = 1;
			_visible = true;
			_name = name;
		}

		public Message(String text, Vector2 position, Color color, string font)
		{
			_text = text;
			_position = position;
			_color = color;
			_font = font;
			_scale = 1;
			_visible = true;
		}

		/// <summary>
		/// Color used for drawing the message
		/// </summary>
		public Color color
		{
			get { return _color; }
			set { _color = value; }
		}

		/// <summary>
		/// position used for drawing the message
		/// </summary>
		public Vector2 position
		{
			get { return _position; }
			set { _position = value; }
		}

		/// <summary>
		/// the text that will be drawn
		/// </summary>
		public String text
		{
			get { return _text; }
			set { _text = value; }
		}

		/// <summary>
		/// the font that will be used to draw the message
		/// </summary>
		public String font
		{
			get { return _font; }
			set { _font = value; }
		}

		/// <summary>
		/// the scale of the message drawn
		/// </summary>
		public float scale
		{
			get { return _scale; }
			set { _scale = value; }
		}

		public bool visible
		{
			get { return _visible; }
			set { _visible = value; }
		}

		public string name
		{
			get { return _name; }
			set { _name = value; }
		}

		/// <summary>
		///  renders the text in this message based on the font
		/// </summary>
		public void render()
		{
			FontManager.getSingleton.get(_font).render(_position, _color, _text, _scale);
		}

	}
}
