/*
 *  Author: Josh Wilson & David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    /*! Represents the scene/world in the game. Currently there is no scene culling.*/
    public class Scene
    {
        protected SortedDictionary<int, Layer> _layers;
        private Vector2 _currentPosition;
        private Camera2D _camera;

        /// <summary>
        /// Constructor.
        /// </summary>
        public Scene()
        {
			_layers = new SortedDictionary<int, Layer>();
            _currentPosition = new Vector2(0, 0);
            _camera = Camera2D.getSingleton;
        }

        /// <summary>
        /// Adds a layer to the scene.
        /// </summary>
        /// <param name="layerName">Name of the layer to add to the scene</param>
        /// <param name="layer">Layer to add to the scene</param>
        public void AddLayer(int layerDepth, Layer layer)
		{
			_layers.Add(layerDepth, layer);
		}

		/// <summary>
		/// Removes a layer from the scene.
		/// </summary>
        /// <param name="layerName">Name of layer to remove from the scene</param>
		public void RemoveLayer(int layerDepth)
		{
			_layers.Remove(layerDepth);
		}

        public Layer getLayer(int layerDepth)
        {
            if (_layers.ContainsKey(layerDepth))
            {
                return _layers[layerDepth];
            }

            return null;
        }

        /// <summary>
        /// Updates the Scene.
        /// </summary>
        public void Update()
        {
            Vector2 displacement = _currentPosition - _camera.Position;

            foreach (Layer thisLayer in _layers.Values)
            {
                //Paralax Scroll each Layer.
				thisLayer.Update(displacement);

                //Cull each layer. Future Implementation.
            }

            _currentPosition = _camera.Position;
        }

        /// <summary>
        /// Renders the Scene by calling Render on each layer.
        /// </summary>
        public void Render(byte type, Effect effect)
        {
            // Hey Josh, you might want to sort this by depth before rendering?
            // instead of giving names to layers you might want to do depth instead
            foreach (Layer thisLayer in _layers.Values)
            {
                thisLayer.Render(type, effect);
            }
        }

        public Dictionary<int, List<Light>> getEffectiveLights()
        {
            Dictionary<int, List<Light>> retVal = new Dictionary<int, List<Light>>(); 

            foreach (KeyValuePair<int, Layer> layerPair in _layers)
            {
                if (layerPair.Value.Lights.Count == 0)
                    continue;

                List<Light> lights = new List<Light>();
                foreach (Light light in layerPair.Value.Lights)
                {
                    lights.Add(light);
                }

                retVal.Add(layerPair.Key, lights);
            }

            return retVal;
        }

		/// <summary>
		/// Culls out parts of the screen that aren't visible in the screen space.
		/// </summary>
		private void Cull()
		{

		}
    }
}
