/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    /*! Composite that renders the scene: diffuse and normal */
    public class SceneComposite: Composite
    {
        protected Color _normalClear;
        protected Color _diffuseClear;

        protected Effect _normalEffect;

		public SceneComposite()
			: base("scene")
        {
            GraphicsCore graphicsCore = GraphicsCore.getSingleton;
    
            _renderMode = PassType.Scene;
            _outputTarget.Add("diffuseTexture", new RenderTarget2D(graphicsCore.Graphics.GraphicsDevice,
                graphicsCore.Width, graphicsCore.Height, 1, SurfaceFormat.Color,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleType,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality));
            _outputTarget.Add("normalTexture", new RenderTarget2D(graphicsCore.Graphics.GraphicsDevice,
                graphicsCore.Width, graphicsCore.Height, 1, 
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.BackBufferFormat,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleType,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality));

			_normalClear = new Color(127, 127, 255, 255);
			_diffuseClear = new Color(new Vector4(.039f, .098f, .165f, 1));
			//_diffuseClear = new Color(new Vector4(1, 1, 1, ));

            _normalEffect = EffectResourceManager.getSingleton.load("assets/shaders/normal");
        }

        /*! \copydoc Composite::render */
        public override void render()
        {
            GraphicsDevice graphicsDevice = GraphicsCore.getSingleton.Graphics.GraphicsDevice;

            // render normal
            graphicsDevice.SetRenderTarget(0, _outputTarget["normalTexture"]);

            // Very Important! Must be this clear color!
            graphicsDevice.Clear(_normalClear);
            
            // scene manager render normal

			//// sprite sort mode must be immediate for effects to work
			//GraphicsCore.getSingleton.SpriteBatch.Begin(
			//    SpriteBlendMode.AlphaBlend, 
			//    SpriteSortMode.Immediate, 
			//    SaveStateMode.None);

            //SpriteManager.getSingleton.renderAll(SpriteMap.NORMAL, _normalEffect);
            SceneManager.getSingleton.Render(SpriteMap.NORMAL, _normalEffect);

			//GraphicsCore.getSingleton.SpriteBatch.End();

            graphicsDevice.SetRenderTarget(0, null);

            // render diffuse
            graphicsDevice.SetRenderTarget(0, _outputTarget["diffuseTexture"]);
            graphicsDevice.Clear(_diffuseClear);

			//// sort mode is backtofront, but we might switch to just a painter's alg.
			//GraphicsCore.getSingleton.SpriteBatch.Begin(
			//    SpriteBlendMode.AlphaBlend, 
			//    SpriteSortMode.Immediate, 
			//    SaveStateMode.None);

            // scene manager render diffuse
            //SpriteManager.getSingleton.renderAll(SpriteMap.DIFFUSE, null);
            SceneManager.getSingleton.Render(SpriteMap.DIFFUSE, null);

            //GraphicsCore.getSingleton.SpriteBatch.End();

            graphicsDevice.SetRenderTarget(0, null);
        }

        public Color DiffuseClear { set { _diffuseClear = value; } }
    }
}
