/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace rhodes.graphics
{
    /*! This class will manage all the sprite entities in the game. You can have multiple entities of the 
        same spritemap, so that you can have the same thing on the screen multiple ug. */ 
    public class SpriteManager
    {
        private static volatile SpriteManager _instance;
        private static object _syncRoot = new Object();
        private SortedDictionary<String, Sprite> _sprites;

        private SpriteManager() 
        {
            _sprites = new SortedDictionary<string, Sprite>();
        }

        /*! Returns singleton instance of the SceneManager */
        public static SpriteManager getSingleton
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncRoot)
                    {
                        if (_instance == null)
                            _instance = new SpriteManager();
                    }
                }

                return _instance;
            }
        }

		/*! Creates a sprite from a sprite map filename and returns that sprite  
		 * \param spriteName The name of the sprite
		 * \param spriteMapFilename The path and filename of the sprite 
		 * \return Returns a sprite */
        public Sprite load(String spriteName, String spriteMapFilename)
        {
            Sprite retVal = null;

            if(_sprites.ContainsKey(spriteName))
            {
                //Console.Error.WriteLine( "Sprite entity: " + spriteName + " already exists." );
                retVal = _sprites[spriteName];
            }
            else
            {
                SpriteMap spriteMap = SpriteResourceManager.getSingleton.load(spriteMapFilename);
                
                if (spriteMap != null)
                {
                    retVal = new Sprite(spriteName, spriteMap);
                    _sprites.Add(spriteName, retVal);
                }
            }

            return retVal;
        }

		/*! Creates a sprite from a sprite map and returns that sprite 
		* \param spriteName The name of the sprite
		* \param spriteMap The data used to create the sprite
		* \return Returns a sprite */
		public Sprite load(String spriteName, SpriteMap spriteMap)
		{
			Sprite retVal = null;

			if (_sprites.ContainsKey(spriteName))
			{
				//Console.Error.WriteLine( "Sprite entity: " + spriteName + " already exists." );
				retVal = _sprites[spriteName];
			}
			else
			{
				if (spriteMap != null)
				{
					retVal = new Sprite(spriteName, spriteMap);
					_sprites.Add(spriteName, retVal);
				}
			}

			return retVal;
		}

        /*! Renders all the sprites in the manager. DEPRECATE */
        public void renderAll(byte type, Effect effect)
        {
            foreach (Sprite sprite in _sprites.Values)
            {
                //sprite.render(type, effect, Vector2.Zero);
            }
        }


        /*! Updates all sprites in the manager.
         * \param elapsedTime The time elapsed since last tick in milliseconds. */
        public void update(float elapsedTime)
        {
            // loop through all entities and update
        }

        /*! Gets a sprite by name.
         * \param name The name of the sprite.
         * \return Returns a sprite. */
        public Sprite get(String name)
        {
            if (_sprites.ContainsKey(name))
            {
                return _sprites[name];
            }

            return null;
        }

        /*! Removes all sprites */
        public void removeAll()
        {
            _sprites.Clear();
        }
    }
}
