using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

	/* 
	 * FileName: TextCharacter.cs
	 * Author:  Anthony Reese
	 * 
	 * Represents a character of a Font
	 */


namespace rhodes.graphics
{
	public class TextCharacter
	{
		private int _imgageID;
		private Vector2 _position;
		private Vector2 _size;
		private int _advanceWidth;
		private int _leftSideBearing;

		/// <summary>
		/// Creates a character with an id, position, size, advanceWidth, and leftSideBearing
		/// </summary>
		/// <param name="id"></param>
		/// <param name="position"></param>
		/// <param name="size"></param>
		/// <param name="advanceWidth"></param>
		/// <param name="leftSideBearing"></param>
		public TextCharacter(int id, Vector2 position, Vector2 size, int advanceWidth, int leftSideBearing)
		{
			_imgageID = id;
			_position = position;
			_size = size;
			_advanceWidth = advanceWidth;
			_leftSideBearing = leftSideBearing;
		}

		/// <summary>
		/// The id of the character
		/// </summary>
		public int imageID
		{
			get { return _imgageID; }
		}

		/// <summary>
		/// The position of the character
		/// </summary>
		public Vector2 position
		{
			get { return _position; }
		}

		/// <summary>
		/// The advancedWidth of the character
		/// </summary>
		public int advancedWidth
		{
			get { return _advanceWidth; }
		}

		/// <summary>
		/// The leftSideBearing of the character
		/// </summary>
		public int leftSideBearing
		{
			get { return _leftSideBearing; }
		}

		/// <summary>
		/// The width of the character
		/// </summary>
		public int width
		{
			get { return (int)_size.X; }
		}

		/// <summary>
		/// The height of the character
		/// </summary>
		public int height
		{
			get { return (int)_size.Y; }
		}

		/// <summary>
		/// The X position of the character
		/// </summary>
		public int xPosition
		{
			get { return (int)_position.X; }
		}

		/// <summary>
		/// The Y position of the character
		/// </summary>
		public int yPosition
		{
			get { return (int)_position.Y; }
		}

	}
}
