/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using RhodesProcessorLib;
using rhodes.graphics.particles.emitters;

namespace rhodes.graphics.particles
{
    /*! Reads & process simple entities */
    public class ParticleReader : ContentTypeReader<ParticleEngine>
    {
        /*! \copydoc ContentTypeReader::Read */
        protected override ParticleEngine Read(ContentReader input, ParticleEngine existingInstance)
        {
            List<EmitterInfo> emitterInfoList = input.ReadObject<List<EmitterInfo>>();
            List<Emitter> emitters = new List<Emitter>();

            foreach (EmitterInfo info in emitterInfoList)
            {
                SpriteResourceManager.getSingleton.load(info.Spritemap);
				if (info.MinVel.Equals(Vector2.Zero) && info.MaxVel.Equals(Vector2.Zero))
				{
					if (info.Positions.Count > 1)
					{
						emitters.Add(new SplineEmitter(info.Positions,
							info.BlendEffect, info.Spritemap, 0, info.ParticlePersSec, info.ParticleAmount,
							info.EmitLifetime, info.ParticleLifetime, info.ParticleLifeRandomizer, info.StartRotationRange,
							info.MinStartRotSpeed, info.MaxStartRotSpeed, info.MinEndRotSpeed, info.MaxEndRotSpeed,
							info.MinStartSize, info.MaxStartSize, info.MinEndSize, info.MaxEndSize, info.MinStartSpeed,
							info.MaxStartSpeed, info.MinEndSpeed, info.MaxEndSpeed, info.MinStartColor, info.MaxStartColor,
							info.MinEndColor, info.MaxEndColor));
					}
					else if (info.Positions.Count == 1)
					{
						emitters.Add(new PointEmitter(info.Positions[0],
							info.BlendEffect, info.Spritemap, 0, info.ParticlePersSec, info.ParticleAmount,
							info.EmitLifetime, info.ParticleLifetime, info.ParticleLifeRandomizer, info.StartRotationRange,
							info.MinStartRotSpeed, info.MaxStartRotSpeed, info.MinEndRotSpeed, info.MaxEndRotSpeed,
							info.MinStartSize, info.MaxStartSize, info.MinEndSize, info.MaxEndSize, info.MinStartSpeed,
							info.MaxStartSpeed, info.MinEndSpeed, info.MaxEndSpeed, info.MinStartColor, info.MaxStartColor,
							info.MinEndColor, info.MaxEndColor));
					}
					else
					{
						Console.Error.WriteLine("BAD EMITTER");
						continue;
					}
				}
				else
				{
					if (info.Positions.Count == 1)
					{
						if (info.MinVel.X > 1 || info.MinVel.Y > 1)
						{
							emitters.Add(new SprayEmitter(info.Positions[0], info.MinVel,
								info.BlendEffect, info.Spritemap, 0, info.ParticlePersSec, info.ParticleAmount,
								info.EmitLifetime, info.ParticleLifetime, info.ParticleLifeRandomizer, info.StartRotationRange,
								info.MinStartRotSpeed, info.MaxStartRotSpeed, info.MinEndRotSpeed, info.MaxEndRotSpeed,
								info.MinStartSize, info.MaxStartSize, info.MinEndSize, info.MaxEndSize, info.MinStartSpeed,
								info.MaxStartSpeed, info.MinEndSpeed, info.MaxEndSpeed, info.MinStartColor, info.MaxStartColor,
								info.MinEndColor, info.MaxEndColor));
						}
						else
						{
							emitters.Add(new DirectionalEmitter(info.Positions[0], info.MinVel, info.MaxVel,
								info.BlendEffect, info.Spritemap, 0, info.ParticlePersSec, info.ParticleAmount,
								info.EmitLifetime, info.ParticleLifetime, info.ParticleLifeRandomizer, info.StartRotationRange,
								info.MinStartRotSpeed, info.MaxStartRotSpeed, info.MinEndRotSpeed, info.MaxEndRotSpeed,
								info.MinStartSize, info.MaxStartSize, info.MinEndSize, info.MaxEndSize, info.MinStartSpeed,
								info.MaxStartSpeed, info.MinEndSpeed, info.MaxEndSpeed, info.MinStartColor, info.MaxStartColor,
								info.MinEndColor, info.MaxEndColor));
						}
					}
					else
					{
						//Console.Error.WriteLine("BAD EMITTER");
						continue;
					}
				}
            }
            
            return new ParticleEngine(emitters);
        }
    }
}
