using System;
using System.Collections.Generic;
using System.Text;
#if !XBOX
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Collections.Specialized;
#else
using System.Reflection;
using System.IO;
#endif

namespace rhodes.scripting
{
    public class Script
    {
#if !XBOX
        private CompilerResults codeResults = null;

        public Script(CompilerResults results)
        {
            codeResults = results;
            return;
        }

        public CompilerResults results
        {
            get
            {
                return codeResults;
            }
            set
            {
                codeResults = value;
            }
        }

        public object run(string class_name, string method, object[] parameters)
        {
            try
            {
                return codeResults.CompiledAssembly.GetType(class_name, true).GetMethod(method).Invoke(null, parameters);
            }
            catch(Exception)
            {
                return null;
            }
        }
#else
        private Assembly asm = null;

        public Script(Assembly _asm)
        {
            asm = _asm;
            return;
        }

        public object run(string class_name, string method, object[] parameters)
        {
            if (!asm)
                return null;
            try
            {
                asm.GetType(class_name, true).GetMethod(method).Invoke(null, parameters);
            }
            catch (Exception)
            {
                return null;
            }
        }
#endif

        public object run(string class_name, string method)
        {
            return run(class_name, method, null);
        }
    }

    public class ScriptEngine
    {
#if !XBOX
        private CSharpCodeProvider codeProvider = null;
        private CompilerParameters codeParameters = null;

        public ScriptEngine()
        {
            codeProvider = new CSharpCodeProvider();
            //codeCompiler = codeProvider.CreateCompiler();
            codeParameters = new CompilerParameters();
            setupInitialParams();
            return;
        }

        protected void setupInitialParams()
        {
            // Set compiler parameters for optimization
            codeParameters.CompilerOptions = "/optimize";
            // We obviously don't want to create an executable
            codeParameters.GenerateExecutable = false;
            // Instead, store it into memory
            codeParameters.GenerateInMemory = true;
            // Don't care about debug information
            codeParameters.IncludeDebugInformation = false;
            // Don't want to treat warnings as errors
            codeParameters.TreatWarningsAsErrors = false;
            return;
        }

        public CompilerParameters parameters
        {
            get
            {
                return codeParameters;
            }
            set
            {
                codeParameters = value;
            }
        }

        public StringCollection references
        {
            get
            {
                return codeParameters.ReferencedAssemblies;
            }
        }

        public void clearReferences()
        {
            codeParameters.ReferencedAssemblies.Clear();
            return;
        }

        public void addReference(string reference)
        {
            codeParameters.ReferencedAssemblies.Add(reference);
            return;
        }

        public void removeReference(string reference)
        {
            codeParameters.ReferencedAssemblies.Remove(reference);
            return;
        }

        public Script loadScript(string filename)
        {
            if(codeParameters == null)
                return null;
            CompilerResults codeResults = codeProvider.CompileAssemblyFromFile(codeParameters, filename);
            return new Script(codeResults);
        }

        public Script loadSource(string source_code)
        {
            if(codeParameters == null)
                return null;
            CompilerResults codeResults = codeProvider.CompileAssemblyFromSource(codeParameters, source_code);
            return new Script(codeResults);
        }
#else
        public Script loadDll(string dll_filename)
        {
            if (!File.Exists(dll_filename))
                return null;
            return new Script(Assembly.LoadFrom(dll_filename));
        }
#endif
    }
}
