
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using RhodesProcessorLib;
using SedohrProcessorLib;
using rhodes.graphics;
using rhodes.graphics.particles;
using rhodes.graphics.particles.emitters;
using rhodes.engine;
using rhodes.physics;
using rhodes.audio;
using rhodes.input;

using sedohr.player;
using sedohr.weapon;
using sedohr.objects;

namespace sedohr
{
	/// <summary>
	/// Creates all the layers and assets for the level from the .level file 
	/// Author: David Huynh
	/// </summary>
    class Level
    {
        private List<LayerInfo> _layerInfo;

		public static ParticleEngine _generatorSmoke;

        public Level()
        {
            _layerInfo = new List<LayerInfo>();
			_generatorSmoke = ParticleManager.getSingleton.load("generatorSmoke", "assets/particles/genSmokeEffect");
        }

        public bool load(String sceneName, ObjectManager objectManager)
        {
            Scene scene = new Scene();

            foreach (LayerInfo layerInfo in _layerInfo)
            {
                Layer layer = new Layer(layerInfo.Proportion);

                foreach (String batch in layerInfo.Statics)
                {
                    SimpleEntity entity = EntityManager.getSingleton.loadSimple(batch.Replace(".simple", ""));
                    layer.AddSprite(entity.Sprite.Name, entity.Sprite);
                    Console.Out.WriteLine("Loaded static geometry: " + batch);
                }

                foreach (String simple in layerInfo.SimpleObjects)
                {
                    SimpleObject simpleObject = objectManager.loadSimple(simple);
					layer.AddSprite(simpleObject.SimpleEntity.Sprite.Name, simpleObject.SimpleEntity.Sprite);
                    Console.Out.WriteLine(simple);
                }

                foreach (String compound in layerInfo.CompoundObjects)
                {
                    CompoundObject compoundObject = objectManager.loadCompound(compound);
                    foreach (SimpleEntity simpleEntity in compoundObject.Compound.Entities)
                    {
                        layer.AddSprite(simpleEntity.Sprite.Name, simpleEntity.Sprite);
						if (simpleEntity.Sprite.Name.Equals("generator_2"))
						{
							_generatorSmoke.register(simpleEntity.Sprite.Name);
							foreach (DirectionalEmitter dEmitter in _generatorSmoke.Emitters)
							{
								dEmitter.IsActive = true;
							}
							layer.AddParticle("generatorSmoke", _generatorSmoke);
						}
						
                    }
                    Console.Out.WriteLine(compound);
                }
                foreach (LightInfo lightInfo in layerInfo.LightInfo)
                {
                    Console.Out.WriteLine(lightInfo.Color);
                    Light light = new Light();
                    light.Position = lightInfo.Position;
                    light.Attenuation = lightInfo.Attenuation;
                    light.Color = lightInfo.Color;
                    light.IsAttenuated = lightInfo.IsAttenuated;

                    layer.AddLight(light);
                }
                foreach (GraphicsInfo graphicsInfo in layerInfo.Graphics)
                {
                    String name = graphicsInfo.Filename.Substring(graphicsInfo.Filename.LastIndexOf('/') + 1).Replace(".spritemap", "");
                    Sprite sprite = SpriteManager.getSingleton.load(name, graphicsInfo.Filename.Replace(".spritemap", ""));
                    sprite.Position = graphicsInfo.Position;
                    layer.AddSprite(sprite.Name, sprite);
                    Console.Out.WriteLine("Loaded graphics: " + graphicsInfo.Filename);
                }

                scene.AddLayer(layerInfo.Depth, layer);
            }

            SceneManager.getSingleton.AddScene(sceneName, scene);
            SceneManager.getSingleton.setActiveScene(sceneName);
            return true;
        }

        public List<LayerInfo> LayerInfo { get { return _layerInfo; } set { _layerInfo = value; } }
    }
}
