/*
 *  Author: Josh Wilson
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.graphics;
using rhodes.physics;
using rhodes.engine;

using sedohr.player;
using sedohr.weapon;

namespace sedohr.objects
{
    /// <summary>
    /// Manager class for all the hull pieces.
    /// </summary>
    class HullManager
    {
		/// <summary>
		/// HullManager singleton
		/// </summary>
		private static volatile HullManager _instance;
		private static object _syncRoot = new System.Object();

        /// <summary>
        /// List of all the hull pieces 
        /// </summary>
        private SortedDictionary<String, Hull> _hullTiles;
        public SortedDictionary<String, Hull> HullTiles
        {
            get { return _hullTiles; }
        }

        /// <summary>
        /// Hull Constructor
        /// </summary>
        public HullManager()
        {
            _hullTiles = new SortedDictionary<string, Hull>();

        }

        /// <summary>
        /// Returns a singleton of the Hullmanager
        /// </summary>
		public static HullManager getSingleton
		{
			get
			{
				if (_instance == null)
				{
					lock (_syncRoot)
					{
						if (_instance == null)
							_instance = new HullManager();
					}
				}
				return _instance;
			}
		}

        /// <summary>
        /// Add a hull piece to the hull manager
        /// </summary>
        /// <param name="hull">Hull piece to add</param>
        public void Add(Hull hull)
        {
            //If the hull isn't already in the list, add it
            if (_hullTiles.ContainsKey(hull.Name))
            {
                Console.Error.WriteLine("Object " + hull.Name + " already exists.");
            }
            else
            {
                _hullTiles.Add(hull.Name, hull);
            }
        }

        /// <summary>
        /// Get a hull piece.
        /// </summary>
        /// <param name="name">Name of the hull piece to get</param>
        /// <returns>The hull piece, null if there isn't one with the given name</returns>
        public Hull getHull(String name)
        {
            if (_hullTiles.ContainsKey(name))
            {
                return _hullTiles[name];
            }

            return null;
        }

        /// <summary>
        /// Updates all the hull pieces in the list
        /// </summary>
        /// <param name="elapsedTime">Time since the last update</param>
        public void update(float elapsedTime)
        {
            //Loops through all the hull pieces and if they are active calls the hull's update
            foreach (Hull hull in _hullTiles.Values)
            {
				if (hull.Entity.Active)
					hull.update(elapsedTime);
            }
        }
    }
}
