/*
 *  Author: Josh Wilson
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */
using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.graphics;
using rhodes.physics;
using rhodes.engine;

using sedohr.player;
using sedohr.weapon;

namespace sedohr.objects
{
	/// <summary>
	/// Contains the Entity for the level laser particles.
	/// </summary>
	class LevelLaser
	{
		/// <summary>
		/// Name of the level laser.
		/// </summary>
		private String _name;
		public String Name
		{
			get { return _name; }
			set { _name = value; }
		}

		/// <summary>
		/// The level laser's entity.
		/// </summary>
		private SimpleEntity _entity;
		public SimpleEntity Entity
		{
			get { return _entity; }
			set { _entity = value; }
		}

		/// <summary>
		/// An instance of the player manager to cycle through the players upon updating.
		/// </summary>
		private PlayerManager _playerManager;

		/// <summary>
		/// An instance of the object manager to cycle through the objects upon updating.
		/// </summary>
		private ObjectManager _objectManager;

		/// <summary>
		/// LevelLaser Constructor
		/// </summary>
		/// <param name="name">Name of the level laser</param>
		/// <param name="playerManager">Instance of the PlayerManager</param>
		/// <param name="objectManager">Instance of the ObjectiManager</param>
		public LevelLaser(string name, PlayerManager playerManager, ObjectManager objectManager)
		{
			_name = name;
			_playerManager = playerManager;
			_objectManager = objectManager;

			//Load the level laser's entity
			_entity = EntityManager.getSingleton.loadSimple("assets/entities/level/" + _name);

			//Create the Collisions geometry for the black hole
			foreach (Geom geom in _entity.PhysicsBody.GeomList)
			{
				geom.CollisionHandler += this.collisionCallback;
			}
		}

		/// <summary>
        /// Updates the level laser.
        /// </summary>
        /// <param name="elapsedTime">Elapsed time in seconds since the last update.</param>
		public void update(float elapsedTime)
		{
		
		}

		/// <summary>
		/// Handles the collision when something collides with the level laser
		/// </summary>
		/// <param name="g1">The level laser geometry</param>
		/// <param name="g2">The geometry of the other object</param>
		/// <param name="contactList">The list of contact points</param>
		/// <returns></returns>
		public bool collisionCallback(Geom g1, Geom g2, ContactList contactList)
		{
			string g2name = g2.Tag.ToString();

			//If the object that collided with the black hole is a player then kill that player
			if (g2name.Equals("player1") || g2name.Equals("player2") || g2name.Equals("player3") || g2name.Equals("player4"))
			{
				_playerManager.Players[g2name].takeDamage(101);
			}
			else if (g2.CollisionCategories == Enums.CollisionCategories.Cat6)
			{
				//Destroy the object and add to captured list
				if (_objectManager.Simples.ContainsKey(g2name))
				{
					_objectManager.Simples[g2name].takeDamage(SimpleObject.MAX_HEALTH + 1);
				}
				//else if (_objectManager.Compounds.ContainsKey(g2name))
				//{
					
				//}
			}

			return true;
		}
	}
}
