/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Text;

using rhodes.graphics;

namespace sedohr.objects
{
	/// <summary>
	/// Manages all the objects in the level (simple and compound)
	/// </summary>
    class ObjectManager
    {
        private SortedDictionary<String, SimpleObject> _simples;
		public SortedDictionary<String, SimpleObject> Simples
		{
			get { return _simples; }
		}

        private SortedDictionary<String, CompoundObject> _compounds;
		public SortedDictionary<String, CompoundObject> Compounds
		{
			get { return _compounds; }
		}

        public ObjectManager()
        {
            _simples = new SortedDictionary<string, SimpleObject>();
            _compounds = new SortedDictionary<string, CompoundObject>();
        }

        public SimpleObject loadSimple(String filename)
        {
			int lastIndex = filename.LastIndexOf('.');
			if (lastIndex != -1)
				filename = filename.Replace(filename.Substring(lastIndex), "");

			String name = filename.Substring(filename.LastIndexOf('/') + 1);
            if (_simples.ContainsKey(name))
            {
                Console.Error.WriteLine("Object already loaded: " + filename);
                return _simples[name];
            }
			
            SimpleObject simple = GraphicsCore.getSingleton.Content.Load<SimpleObject>(filename);
            _simples.Add(name, simple);

			return simple;
        }

		public void update(float elapsedTime)
		{
			// remember to call physics update before this
			// update all dynamic simples
			foreach (SimpleObject entity in _simples.Values)
			{
				entity.update(elapsedTime);
			}
		}

        public CompoundObject loadCompound(String filename)
        {
            String name = filename.Substring(filename.LastIndexOf('/') + 1);

            int lastIndex = filename.LastIndexOf('.');
            if (lastIndex != -1)
                filename = filename.Replace(filename.Substring(lastIndex), "");

            if (_compounds.ContainsKey(name))
            {
                Console.Error.WriteLine("Object already loaded: " + filename);
                return _compounds[name];
            }

            CompoundObject compound = GraphicsCore.getSingleton.Content.Load<CompoundObject>(filename);
            compound.Parent = this;
            _compounds.Add(name, compound);

			return compound;
        }

        public void addSimple(SimpleObject simple)
        {
            if (_simples.ContainsKey(simple.Name))
            {
                Console.Error.WriteLine("Object " + simple.Name + " already exists.");
            }
            else
            {
                _simples.Add(simple.Name, simple);
            }
        }

        public SimpleObject getSimple(String name)
        {
            if (_simples.ContainsKey(name))
            {
                return _simples[name];
            }

            return null;
        }

        public CompoundObject getCompound(String name)
        {
            if (_compounds.ContainsKey(name))
            {
                return _compounds[name];
            }

            return null;
        }

        public void removeCompound(String name)
        {
            if (_compounds.ContainsKey(name))
            {
                _compounds.Remove(name);
            }
            else
            {
                Console.Error.WriteLine("Compound Object " + name + " doesn not exist.");
            }
        }
    }
}
