using System;

/*
 * EventFilter.cs
 * 
 * Author: Anthony Reese 
 * 
 * Interface for event filters that an eventlistener will use to mange events
 */

namespace sedohr.player
{
	public interface EventFilter
	{
		/// <summary>
		/// Regisers an event tht this filter will subscribe to.
		/// </summary>
		/// <param name="eventName"></param>
		void register(String eventName);

		/// <summary>
		/// Regisers an event tht this filter will subscribe to and a throttle.
		/// Throttle is the maximum number of events per event class that can be
		/// processed by the generator within a quantum period.
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="throttle"></param>
		void register(String eventName, float throttle);

		/// <summary>
		/// Unregisers an event this filter will subscribe to
		/// </summary>
		/// <param name="eventName"></param>
		void unregister(String eventName);

		/// <summary>
		/// Unregisters all events this filter is subscribing to
		/// </summary>
		void unRegisterAll();
	}
}