/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.engine;
using rhodes.graphics;
using rhodes.input;

using sedohr.weapon;


namespace sedohr.player
{
	/// <summary>
	/// Abstract class that all player states extend from
	/// </summary>
    abstract class PlayerState
	{
		#region Attributes

		/// <summary>
		/// The player object
		/// </summary>
        protected Player _player;

		/// <summary>
		/// The name of the player
		/// </summary>
		protected string _name;

		#endregion

		#region Constructor

		/// <summary>
		/// The PlayerState constructor
		/// </summary>
		/// <param name="player">The player that this state refers to</param>
		public PlayerState(Player player)
        {
            _player = player;
		}

		#endregion

		#region Functions

		/// <summary>
		/// Function that will be used to update the state
		/// </summary>
		/// <param name="elapsedTime">The amount of time that has passed</param>
		public abstract void update(float elapsedTime);

		/// <summary>
		/// Function that will be used to handle logic, input, etc.
		/// </summary>
        public abstract void handle();

		/// <summary>
		/// Function that will be called when the player enters the state
		/// </summary>
        public abstract void enter();

		/// <summary>
		/// Function that will be called when the player exits the state
		/// </summary>
        public abstract void exit();

		/// <summary>
		/// Function that is run when a collision occurs
		/// </summary>
		/// <param name="g1">The geometry that is being hit</param>
		/// <param name="g2">The geometry that hit the first</param>
		/// <param name="cl">The list of contacts</param>
		/// <returns>Returns true</returns>
        public abstract bool collisionCallback(Geom g1, Geom g2, ContactList cl);

		/// <summary>
		/// The callback that is run when the animation is switched
		/// </summary>
		/// <param name="animation">The name of the animation</param>
        public abstract void animationCallback(String animation);

		#endregion

		#region Accessors

		/// <summary>
		/// Gets or sets the player object tha this state refers to
		/// </summary>
		public Player Player
		{
			get
			{
				return _player;
			}
			set
			{
				_player = value;
			}
		}

		/// <summary>
		/// Gets or sets the player's name
		/// </summary>
		public string Name
		{
			get
			{
				return _name;
			}
			set
			{
				_name = value;
			}
		}

		#endregion
	}
}
